// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2017.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
// Lieux (et pseudo-lieux) de Black de systèmes classiques, avec retard pur, ou lieux définis par trois (matrices) constantes.
// On utilise la fonction [fr,db,phi]=dbphifr( ), en particulier pour tracer les pseudo-lieux, ou dpf_cd.sci.
// Black diagram for continuous or sampled systems with or without delay, or Black diagram for three constant matrice.
// We can plot Black diagramm for "pseudo-lieux", in this case s= w*exp(%i*teta), ( 0<teta<180°). 
// DEPENDENCIES : only dbphifr.sci or dpf_cd.sci (Autoelem Toolbox).
// ====================================================================
function bblack(varargin)
//Change "bblack" to "black" if you can do : "cacsdlib.black" in scilab-6.0.0
rhs = size(varargin);
sys = varargin(1);
//cacsdlib.black   see line two.
if and(typeof(sys) <> ["state-space","rational","zpk","rd","constant"]) then
  msg = _("Wrong type for argument #%d: rational, state-space, iodelay system or 3 matrice of constants expected.\n");
  error(msprintf(msg, 1))
  return;
end
if rhs==1 then //Un système seul est envisagé.
   if type(sys) <> 1 then
     [frq,db,phi] = dbphifr(sys,1d-3,1d3);
     black(frq,db,phi);
   end
else
  if typeof(sys) == "constant" then
    if rhs < 3 then
      error(msprintf(_("%s: Wrong size for inputs arguments #%d: 3 matrice of constants expected.\n"),"black",1));
    end
    black(varargin(:));
  else
    der = varargin($);avder = varargin($-1);
    if type(der) == 10 & type(avder) == 10 then //Le dernier argument et l'avant dernier sont des chaînes de caractères.
      comments = der;//Le dernier argument est pris pour une légende.
      [frq,db,phi] = dbphifr(varargin(1:$-1));
      black(frq,db,phi,comments);
    elseif type(der) == 10 & type(avder) <> 10 then //Ici le dernier argument est soit une légende soit un vecteur d'angles.
      [teta,ieer] = evstr(der); //Test pour savoir si c'est une légende ou un vecteur de réels.
      if ieer == 0 & type(teta) == 1 then //L'évaluation est correcte, et c'est un vecteur de réels.
        [frq,db,phi] = dbphifr(varargin(1:$));
        black(frq,db,phi);
      else //Ce n'est pas un vecteur de réels.
        comments = der; //On le prend pour une légende.
        [frq,db,phi] = dbphifr(varargin(1:$-1));
        black(frq,db,phi,comments);
      end
    else //Pas de légende, pas de vecteur de scalaires. 
      [frq,db,phi] = dbphifr(varargin(1:$));
      black(frq,db,phi);
    end
  end
end
//rajouter des éléments graphiques
fig = gcf();
fig.background = 8;
a = gca();
A = a.data_bounds;
xpoly(A(:,1),[0;0]); //axe 0 db.
xpoly([-180;-180],A(:,2)); //axe -180.
a.title.text = "Courbe de Black";a.title.font_style = 4;a.title.font_size = 2;
a.grid(1:2) = 13;
a.y_label.text = "Gain en décibels";a.x_label.text = "Phase en degrés";
CH = a.children;
if CH(3).type == "Legend" then
   CH(3).legend_location = "in_upper_left";
end
CH(1:2).foreground = 24;
endfunction
