// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2017.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
// Lieux de Bode de systèmes classiques, avec retard pur, ou lieux définis par trois matrices constantes.
// On utilise la fonction [fr,db,phi]=dbphifr( ), en particulier pour tracer les pseudo-lieux ou dpf_cd.sci.
// Bode diagram for continuous or sampled systems with or without delay, or Bode diagram for three constant matrice.
// We can plot Bode diagramm for "pseudo-lieux", in this case s= w*exp(%i*teta), ( 0<teta<180°). 
// DEPENDENCIES : only dbphifr.sci or dpf_cd.sci (Autoelem Toolbox).
// ====================================================================
function bbode(varargin)
//Change "bbode" for "bode" if you can do : "cacsdlib.bode" in scilab-6.0.0
rhs = size(varargin);
sys = varargin(1);
//cacsdlib.bode     see line two.
if and(typeof(sys) <> ["state-space","rational","zpk","rd","constant"]) then
  msg = _("Wrong type for argument #%d: rational, state-space, iodelay system or 3 matrices of constants expected.\n");
  error(msprintf(msg, 1))
  return;
end
if rhs==1 then //Un système seul est envisagé.
   if type(sys) <> 1 then
     [frq,db,phi] = dbphifr(sys,1d-3,1d3);
     bode(frq,db,phi);
   end
else
  if typeof(sys) == "constant" then
    if rhs < 3 then
      error(msprintf(_("%s: Wrong size for inputs arguments #%d: 3 matrices of constants expected.\n"),"bode",1));
    end
    bode(varargin(:));
  else
    der = varargin($);avder = varargin($-1);
    if type(der) == 10 & type(avder) == 10 then //Le dernier argument et l'avant dernier sont des chaînes de caractères.
      comments = der;//Le dernier argument est pris pour une légende.
      [frq,db,phi] = dbphifr(varargin(1:$-1));
      bode(frq,db,phi,comments);
    elseif type(der) == 10 & type(avder) <> 10 then //Ici le dernier argument est soit une légende soit un vecteur d'angles.
      [teta,ieer] = evstr(der); //Test pour savoir si c'est une légende ou un vecteur de réels.
      if ieer == 0 & type(teta) == 1 then //L'évaluation est correcte, et c'est un vecteur de réels.
        [frq,db,phi] = dbphifr(varargin(1:$));
        bode(frq,db,phi);
      else //Ce n'est pas un vecteur de réels.
        comments = der; //On le prend pour une légende.
        [frq,db,phi] = dbphifr(varargin(1:$-1));
        bode(frq,db,phi,comments);
      end
    else //Pas de légende, pas de vecteur de scalaires. 
      [frq,db,phi] = dbphifr(varargin(1:$));
      bode(frq,db,phi);
    end
  end
end
//rajouter des éléments graphiques
a = gca();
a.title.text = "Courbes de Bode";a.title.font_style = 4;a.title.font_size = 2;
f = gcf();
f.background = 8;
f.children.grid = [13,13];
f.children(2).y_label.text = "Gain en décibels";
f.children(2).x_label.text = "Fréquences en hertz";
f.children(1).y_label.text = "Phase en degrés";
f.children(1).x_label.text = "Fréquences en hertz";
endfunction
