// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
// Bode factorisation for SIMO, continuous systems (with or without delays), OR sampled systems.
// DEPENDENCIES : only mroots.sci (Autoelem Toolbox).
// ====================================================================
function [k,l,d,tn,td,fdn,fdd] = bodfact(r,flag)
rhs = argn(2);
//
//*****************Tests sur le nombre d'arguments d'entrée.Input arguments test.
//
select rhs
  case 1 then
    flag = "no";
  case 2 then
    if type(flag) == 10 then
      if part(flag,1) <> "y"  then
        flag= "no";
      end
    else
      flag = "no";
    end
end
//
//*****************Fin des tests sur le nombre d'arguments d'entrée.End of inputs arguments test.
//
//*****************Tests sur le type d'arguments d'entrée.
typ = type(r);
select typ
  case 1 then //évident.
    k = r; l = 0; d = 0; tn = 1; td = 1; fdn = []; fdd =[];
    return
  case 2 then //on ramène le vecteur polynôme à un système linéaire continu.
    var = varn(r);
    r = syslin("c",r,poly(1,var,"c"));
    d = 0; dom = "c";
  case 16 then
    d = 0;
    if r(1)(1) == "lss" then
      r = clean(ss2tf(r));//Voir exemple dans ss2tf.
    end
    dom = r.dt;
    if dom == [] then
       r = syslin("c",r);
       d = 0; dom = "c";
    end
    if dom == "d" then
      dom = 1;
    end
  case 17 then
    if typeof(r) == "zpk" then
      r = zpk2tf(r);
      dom = r.dt;
      if dom == "d" then
        dom = 1;
      end
    end
    if typeof(r) == "rd" then
      d = r.iodelay; //extraction du retard pur pour système continu.
      r = r.H;//extraction de la transmittance.  
      dom = r.dt;
    else
      error(msprintf(_("%s: Wrong type for input argument #%d: iodelay expected.\n"),"bodfact",1));
    end
end
//
//*****************Fin des tests sur le type d'arguments.
//
//*****************Les différentes formes de la factorisation (systèmes continus et échantillonnés).
//
simp_mode(%t); //Au cas où r = syslin("?",num,den) avec un terme commun entre "num" et den".
r = simp(r); //On force la simplification : bodfact est utilisé en fréquentiel.
eps = 1.e-14;
num = r(2);den = r(3);
var = varn(den);
//
//*****************Le numérateur : racines nulles et racines == 1 et premier coefficient non nul.
//
[Rmn, racn] = mroots(num);racn = clean(racn,eps);//mroots.sci obligatoire,
coen = coeff(num);
nn = min(find(coen <> 0))-1;//Nombre de racines nulles
an0 = coen(nn+1);//Premier coefficient non nul.
racnr = real(racn(imag(racn) == 0)); //Les racines réelles à 1.e-14 près.
racni = racn(imag(racn) > 0); //Les racines imaginaires au dessus axe horizontal. 
racnun = racni(abs(racni) < (1+100*eps) & abs(racni)>(1-100*eps));
//Les racines dont le module est compris dans la couronne  [1+100*eps, 1-100*eps].
rnun=length(racnr(racnr<(1+100*eps) & racnr>(1-100*eps))); //Le nombre de racines considérées comme égales à 1.
//
//*****************Le dénominateur : racines nulles et racines == 1 et premier coefficient non nul.
//
[Rmd, racd] = mroots(den);racd = clean(racd,eps);
coed = coeff(den);
nd=min(find(coed <> 0))-1;//Nombre de racines nulles.
ad0 = coed(nd+1); //Premier coefficient non nul.
racdr = real(racd(imag(racd) == 0)); //Les racines réelles.
racdi = racd(imag(racd) > 0); //Les racines imaginaires au dessus de l'axe horizontal.
racdun = racdi(abs(racdi) < (1+100*eps) & abs(racdi)>(1-100*eps));
//Les racines dont le module est compris dans la couronne  [1+100*eps, 1-100*eps].
rdun=length(racdr(racdr<(1+100*eps) & racdr>(1-100*eps))); //Le nombre de racines considérées comme égales à 1.
//
//*****************Calcul de k, l et fréquences (fdn (zéros), fdd (pôles) sur axe imaginaire) pour les systèmes continus.
//
if dom == "c" then //Cas continu
  k = an0/ad0; //Le gain statique pour systèmes continus ou des rationnels.
  l = nn - nd; //Nombre de facteurs de la forme s**l (dérivateurs si l > 0 ,intégrateurs si l < 0).
  fdn = (1/(2*%pi))*imag(racni(real(racni) == 0)); //Fréquences donnant des zéros sur l'axe imaginaire positif.
  fdd = (1/(2*%pi))*imag(racdi(real(racdi) == 0)); //Fréquences donnant des pôles sur l'axe imaginaire positif.
end
//
//*****************Calcul de k, l, d et fréquences (fdn, fdd) sur le cercle de rayon 1 pour les systèmes échantillonnés.
//
if  type(dom) == 1 then //Cas discret, ou échantillonné.
  l = rnun - rdun; // Nombres de facteurs en (z-1)**l dans la transmittance.
  d = nn - nd; //(-d) retards pour systèmes discrets ou échantillonnés. 
  k = horner(r/poly([-1,1],var,"c")**l,1); //Gain statique pour systèmes discrets ou échantillonnés.
  fdn = (1/(2*%pi*dom))*atan(imag(racnun),real(racnun));//Fréquences donnant des zéros sur le cercle de rayon 1.
  fdd = (1/(2*%pi*dom))*atan(imag(racdun),real(racdun));//Fréquences donnant des pôles sur le cercle de rayon 1.
end
//
//*****************Calcul de tn et td (la factorisation).
//
flag1 = part(flag,1); flag2 = part(flag,2);
//
//******************Factorisation incomplète (systèmes continus ou échantillonnés).
//
select flag1
  case "n" then //pas de factorisation complète.
    if dom == "c" then //Systèmes continus.
      sys = r/poly(0,var)**l; //Elimination des pôles ou zéros à l'origine.
      tn = sys.num/an0;
      td = sys.den/ad0;
    else //échantillonné ou discret. 
      sys = r/(k*poly([-1,1],var,"c")**l); //Elimination des pôles ou zéros en 1.
      sys = sys/poly(0,var)**d; //Elimination des retard purs.
      tn = sys.num;
      td = sys.den;
      coe = coeff(td);
      coelast = coe($);
      tn = tn/coelast;
      td = td/coelast; //Le terme de plus haut degré du dénominateur vaut 1 (td unitaire).
    end
//
//*****************Factorisation complète pour systèmes continus seulement.
//
  case "y" then
    if dom == "c" then
      racn = racn(racn <> 0); //On a déja calculé le nombre de racines nulles du numérateur.
      racd = racd(racd <> 0); //On a déja calculé les nombre de racines nulles du dénominateur.
//
//*****************Factorisation sur le corps des réels.
//
     if flag2 == "2" then //factorisation complète : termes premier et/ou second ordre (factorisation sur R).    
//*********Le numérateur.     
       racnr = racnr(racnr <> 0); //Racines réelles non nulles.
       taun = -(1)./racnr; //Les constantes de temps des premiers ordres.
       prn = ones(taun) + poly(0,var)*taun; //Calcul des polynômes du premier degré.
       prni = (poly(0,var).^2)*(ones(racni)./(abs(racni).^2)) - 2*poly(0,var)..
       *(real(racni)./(abs(racni).^2)) + ones(racni); //Et du second degré.
//*********Le dénominateur.     
       racdr = racdr(racdr <> 0); //Racines réelles non nulles.
       taud = -(1)./racdr; //Les constantes de temps des premiers ordres.
       prd = ones(taud) + poly(0,var)*taud; //Calcul des polynômes du premier degré.
       prdi = (poly(0,var).^2)*(ones(racdi)./(abs(racdi).^2)) - 2*poly(0,var)..
       *(real(racdi)./(abs(racdi).^2)) + ones(racdi); //Et du second degré.
       tn = [prn;prni]; //Regroupement des termes du numérateur.
       td = [prd;prdi]; //Regroupement des termes du dénominateur.
       tn(tn == []) = 1;
       td(td == []) = 1;   
     else
//
//*****************Factorisation sur le corps des complexes.
//*****************Factorisation complète avec des termes du premier degré : polynômes complexes éventuellement.
//
        taun = -(1)./racn; //Les constantes de temps des premiers ordres réelles ou complexes du numérateur.
        taud = -(1)./racd; //Idem dénominateur.
        prn = ones(taun) + poly(0,var)*taun; //Calcul des polynômes du premier degré (réels et/ou complexes).
        prd = ones(taud) + poly(0,var)*taud; //Idem dénominateur.
        tn = prn;
        td = prd;
        tn(tn == []) = 1;
        td(td == []) = 1;
     end   
   else
     error(msprintf(_("%s: Wrong type for input argument #%d: Continuous time system expected.\n"),"bodfact",1));
   end
end
endfunction









