// Copyright (C) - lucien.povy@free.fr.  Carantec 2014-2015; 2017 for "zpk".
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
// ====================================================================
// We compute frequencies, magnitudes, phases of SIMO, continuous systems (with or without delays) OR sampled systems.
// We can compute magnitudes, phases for "pseudo-lieux", in this case s= w*exp(%i*teta), ( 0<teta<180°). 
// ==================================================================== 
function [fr,db,phi,splitf] = dbphifr(sys,fmin,fmax,pas,angle)
//
//*****************Initialisation.
pas_def = "auto";
eps = 1.e-14;
l10 = log(10);
[lhs,rhs] = argn(0);
typ = typeof(sys);
drap = 0;
//
//*****************Fin initialisation.
//*****************Tests sur le type d'arguments d'entrée
//
if and(typ <> ["state-space","rational","zpk","rd"]) then
  error(msprintf(gettext("%s: Wrong type for input argument #%d: Linear dynamical system expected.\n"),"dbphifr",1))
end
select typ
  case "state-space" then
    sys = clean(ss2tf(sys));//Voir manuel ss2tf.
    dom = sys.dt;
  case "zpk" then
    sys = zpk2tf(sys);
    dom = sys.dt;
  case "rd" then //cas "rd" : "iodelay toolbox".
    dom = sys.H.dt
  case "rational" then  
    dom = sys.dt //cas 'rational'.
end
if dom == []|dom == 0 then error(96,1), end
if dom == "d" then dom = 1, end
//
//*****************Fin des tests sur le type d'arguments d'entrée.
//
//*****************Discretization (Voir repfreq et repfreqrd) : en partie copié sur les programmes de Serge Steer.
select  rhs
  case 1 then
    pas = pas_def;
    if dom == "c" then 
      fmax = 1.d3; 
    else 
      fmax = 1/(2*dom);
    end
    fmin = eps;
  case 2 then
    if type(fmin) == 10 then
      rhs = 1;
      pas = pas_def;
      if dom == "c" then
        fmax = 1.d3;
        if fmin == "sym" then
          fmin = -fmax;
        else
          if size(fmin,'*') <> size(sys,'*') then //der doit être un vecteur colonne de caractères de même dimension que 'sys'.
            error(mprintf(_("%s: Wrong size for last input argument : same size as system.\n"),"dbphifr"));
          end
          [teta,ieer] = evstr(fmin);
          if ieer == 0 then
            drap = 1;
          else
            error(mprintf(_("%s: Wrong syntax for last input argument : evaluation must give a vector of scalars.\n"),"dbphifr"));
          end
          fmin = eps;
        end
      else
        fmax = 1/(2*dom);fmin = eps;
      end
    else
      frq = fmin;
      if frq(1,1) == 0 then
       frq(1,1) = eps;
      end;
    end
  case 3 then
    if type(fmax) == 10 then
      rhs = 2;
      if size(fmax,'*') <> size(sys,'*') then //der doit être un vecteur colonne de caractères de même dimension que 'sys'.
        error(mprintf(_("%s: Wrong size for last input argument : same size as system.\n"),"dbphifr"));
      end
      [teta,ieer] = evstr(fmax);
      if ieer == 0 then
        drap = 1;
        frq = fmin;
        if frq(1,1) == 0 then
          frq(1,1) = eps;
        end;
      else
       error(mprintf(_("%s: Wrong syntax for last input argument : evaluation must give a vector of scalars.\n"),"dbphifr"));
      end
    elseif type(fmax) == 1 then
      pas = pas_def;
      if type(dom) == 1 then
        fmax = min(fmax,1/(2*dom));
      end
    else
      error(mprintf(_("%s: Wrong type for last input argument :  type 1 or 10 expected.\n"),"dbphifr"));
    end
  case 4 then
    if type(pas) == 10 then
      rhs == 3;
      if size(pas,'*') <> size(sys,'*') then //der doit être un vecteur colonne de caractères de même dimension que 'sys'.
        error(mprintf(_("%s: Wrong size for last input argument : same size as system.\n"),"dbphifr"));
      end
      [teta,ieer] = evstr(pas);
      if ieer ==0 then
        drap = 1;
        pas = pas_def;
      else
        error(mprintf(_("%s: Wrong syntax for last input argument : evaluation must give a vector of scalars.\n"),"dbphifr")); 
      end
    elseif type(pas) == 1 then 
      if fmin == 0 then
        fmin = eps;
      end   
    else
      error(mprintf(_("%s: Wrong type for last input argument :  type 1 or 10 expected.\n"),"dbphifr"));
    end
  case 5 then
    if type(angle) <> 10 then
      error(mprintf(_("%s: Wrong type for last input argument : type 10 expected.\n"),"dbphifr"));
    else
      rhs == 4;
      if size(angle,'*') <> size(sys,'*') then //angle doit être un vecteur colonne de caractères de même dimension que 'sys'.
        error(mprintf(_("%s: Wrong size for last input argument : same size as system.\n"),"dbphifr"));
      end
      [teta,ieer] = evstr(angle);
      if ieer == 0 then
        drap = 1;
      else
        error(mprintf(_("%s: Wrong syntax for last input argument #%d: evaluation must give a vector of scalars.\n"),"dbphifr"));
      end
    end
end
//
splitf = 1;
if rhs <> 2 then
  if type(pas) == 1 then
    splitf = 1
    if fmin < 0 & fmax >= 0 then
      frq = -[exp(l10*((log(eps)/l10):pas:(log(-fmin)/l10))),-fmin];
      if fmax > eps then
        frq1 = [exp(l10*((log(eps)/l10):pas:(log(fmax)/l10))),fmax];
        frq = [frq($:-1:1),frq1];
      else
        frq=frq($:-1:1);
      end
    elseif fmin < 0 & fmax < 0 then
      frq= [exp(l10*((log(-fmax)/l10):pas:(log(-fmin)/l10))),-fmin];
      frq=-frq($:-1:1);
    elseif fmin >= fmax then
      error(msprintf(gettext("%s: Wrong value for input arguments #%d and #%d: %s < %s expected.\n"),..
      "dbphifr",2,3,"fmin","fmax"));
    else
      fmin = max(eps,fmin);
      frq = [exp(l10*((log(fmin)/l10):pas:(log(fmax)/l10))),fmax];
    end
  else
    if typeof(sys) == "rd" then
      [frq,bnds,splitf] = calfrqrd(sys,fmin,fmax);
    else
      [frq,bnds,splitf] = calfrq(sys,fmin,fmax);
    end
  end;
end
//
//*****************Fin de la discrétisation de la fréquence.
//
//*****************Calcul du gain, de la phase des systèmes.(see "bodefact" program)
//
[ligns,colns] = size(sys);
if colns<>1 then
  error(95,1)
end
phi = [];
db = [];
for i = 1:ligns
  sysi = sys(i,1);
  select dom
    case "c" then
      if drap == 1 then
        tetai = teta(i,1);
        if type(tetai) == 1 then
          w = 2*%pi*exp((%pi/180)*%i*tetai)*frq;
        end
      else
        w = 2*%pi*%i*frq;
      end
      [Ki,Li,Fi,TNi,TDi] = bodfact(sysi,"y");//Factorisation complète de Bode.
//
//*****************Calcul gain et phase de Ki, de Li, de Fi.
//
      dbKi = log(abs(Ki));//Le gain Ki.
      if Ki > 0 then
        phiKi = 0;
      else
        phiKi = -180;
      end
      dbLi = Li*log(abs(w));//Les intégrateurs Li et les retard Fi.
      if drap == 1 then //tetai a une valeur différente de 90°.
        phiLi = Li*tetai;
        dbFi = -2*%pi*Fi*cos((%pi/180)*tetai)*frq;
        phiFi = - 360*Fi*sin((%pi/180)*tetai)*frq;
      else //tetai vaut 90°.
        phiLi = Li*90;
        dbFi = 0;
        phiFi =-360*Fi*frq;
      end
//
//*****************Calcul gain et phase de TNi et TDi.//TNi polynôme réel et/ou complexe de degré un.
//
      coTNi = coeff(TNi);
      if TNi == 1 then
        resTNi =  ones(w);
      else
        resTNi = ones(TNi)*ones(w)+coTNi(:,2)*w; //on ne peut utiliser "freq", je ne veux pas "horner".
      end
      coTDi = coeff(TDi); //TDi polynôme réel et/ou complexe de degré un.
      if TDi == 1 then
        resTDi = ones(w);
      else
        resTDi = ones(TDi)*ones(w)+coTDi(:,2)*w; //on ne peut utiliser "freq".
      end
      dbTNi = sum(log(abs(resTNi)),1);
      dbTDi = sum(log(abs(resTDi)),1);
      phiTNi = sum(atand(imag(resTNi),real(resTNi)),1);
      phiTDi = sum(atand(imag(resTDi),real(resTDi)),1);
//
//*****************Gain et phase de chaque système.
//
      dbi = (20/l10)*(dbKi+dbLi+dbFi+dbTNi-dbTDi);
      phii = phiKi+phiLi+phiFi+phiTNi-phiTDi;
//
  else
//cas discret ou échantillonné le retard est pris en compte par la constante Fi.
      flim = 1/(2*dom);//Il peut avoir autrement une erreur si on donne le vecteur fréquence.
      if frq($) > flim then
        frq = frq(find(frq < = flim));
        warning(msprintf(gettext("%s: Frequencies beyond Nyquist frequency are ignored.\n"), "dbphifr"));
      end
      w = exp(2*%pi*%i*dom*frq);
      [Ki,Li,Fi,TNi,TDi] = bodfact(sysi);//Pas de factorisation complète.
      ai = dom*frq;
      dbKi = log(abs(Ki));
      if Ki > 0 then 
        phiKi = 0;
      else
        phiKi = -180;
      end
        resTi = freq(TNi,TDi,w);
        phiTi = phasemag(resTi);
        dbi = (20/l10)*(dbKi+Li*log(abs(2*sin(%pi*ai)))+log(abs(resTi)));
        phii = phiKi + 90*Li + 180*(2*Fi +Li)*ai + phiTi;
  end
  phi = [phi; phii];
  db = [db; dbi];
end
phi = clean(phi,10*eps); //pour éviter des erreurs d'arrondis.
db = clean(db,10*eps); //pour éviter des erreurs d'arrondis.
fr = frq;
//Représentation
if lhs ==1 then 
  fr = db;
end
if lhs ==2 then 
  fr = db; db = phi;
end
endfunction
