// Copyright (C) - lucien.povy@free.fr. Carantec 2017.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
//Discrétisation (Ts période d'échantillonnage), avec bloqueur d'ordre zéro, d'un   système continu retardé.
//Discretization (Ts sampling period) of delayed linear system with Z.H.O.
// ====================================================================
function sl_rd = dscr_sisord(sl,Ts)
//*****************Input arguments tests.
[lhs, rhs] = argn(0);
if type(Ts)<>1 then
  error(msprintf(gettext("%s: Wrong type for input argument #%d: A real expected.\n"),"dscr_sisord",2))
end
if size(Ts,"*")<>1 then
  error(msprintf(gettext("%s: Wrong size for input argument #%d: A scalar expected.\n"),"dscr_sisord",2))
end
if ~isreal(Ts) then
  error(msprintf(gettext("%s: Input argument #%d must be real.\n"),"dscr_sisord",2))
end
if Ts <= 0 then 
  error(msprintf(gettext("%s: Input argument #%d must be strictly positive.\n"),"dscr_sisord",2))
end
select typeof(sl)
  case "rational" then
    sl_rd = ss2tf(dscr(sl,Ts));
    return
  case "zpk" then
    sl = zpk2tf(sl);
    sl_rd = ss2tf(dscr(sl,Ts));
    return
  case "state-space" then
    sl_rd = ss2tf(dscr(sl,Ts));
    return
  case "rd" then
    tol = 1.e-4;
    delay = sl.iodelay;
    sl = sl.H;
    sl=tf2ss(sl);
    [A,B,C,D,x0,dom] = sl(2:7);
    [n1,m1]=size(B);
    if m1 <> 1 then
      error(msprintf(gettext("%s: Input argument #%d must be a linear SISO system.\n"),"dscr_sisord",1))
      return
    end
    dsdom =tol*round((delay/Ts)/tol);
    k = floor(dsdom);//retard entier inférieur.
    alp = dsdom - k;//la reliquat.
    z = %z;
    if alp <= tol then
      sl_rd = ss2tf(dscr(sl,Ts));
      sl_rd = sl_rd/z^k + D;
      sl_rd.dt = Ts;
      return
    end
end
//*****************End of input arguments tests.
//*****************The algorithm.
disc1malp = expm([A,B;0*ones(1,n1+1)]*(1-alp)*Ts);//exponentielle matrice avec dt=(1-alp)*Ts.
phi1malp = disc1malp(1:n1,1:n1);//contribution état initial sur [0,1-alp]Ts.
psi1malp = disc1malp(1:n1,n1+1);//contribution entrée u sur [0,1-alp]Ts.
discalp = expm([A,B;0*ones(1,n1+1)]*alp*Ts);//exponentielle matrice avec dt=alp*Ts.
phialp = discalp(1:n1,1:n1);
psialp = discalp(1:n1,n1+1);
phi = phi1malp*phialp;//Matrice état échantillonné sur intervalle Ts.
phipsialp = phi1malp*psialp;
//calcul du terme du dénominateur.
den = poly(phi,"z");//poly caractéristique de phi.
//calcul de la première transmittance.
G1 = poly(phi - phipsialp*C,"z")/den - 1;
//Calcul de la deuxième transmittance.
G2 = poly(phi - psi1malp*C,"z")/den - 1;
//Résultat.
sl_rd = G1/z + G2;
sl_rd = sl_rd/z^k +D;
sl_rd.dt = Ts;
endfunction

  
