// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
// On retarde un vecteur signal f ([k,l]=size(f)) de d. Delayed functions "f" (delay==d).
// ====================================================================
function [fd,tt] = fdelay(f,t,d)
//
//*****************Input arguments tests.
[lhs, rhs] = argn(0);
if rhs ~= 3 then
  error(msprintf(_("%s: Wrong size for input arguments %d: 3 arguments expected.\n"),"fdelay"))
end
if type(f)<>1|~isreal(f) then 
  error(msprintf(_("%s: Wrong type for argument %d: Real matrix expected.\n"),"fdelay",1))
end
if type(t)<>1|~isreal(t)|and(size(t)>1) then 
  error(msprintf(_("%s: Wrong type for argument %d: Real vector expected.\n"),"fdelay",2))
end
if or(diff(t)<=0) then
  error(msprintf(_("%s: Elements of %dth argument must be in increasing order.\n"),"fdelay",2))
end
if size(f,"c")<>size(t,"*") then
  error(msprintf(_("%s: Incompatible input arguments #%d and #%d: Same sizes expected.\n"),"fdelay",1,2))
end
if type(d)<>1|~isreal(d)|or(size(d)<>1)|d<=0 then 
  error(msprintf(_("%s: Wrong type for argument %d: Real positive scalar expected.\n"),"fdelay",3))
end
//
//*****************End of input arguments tests.
//
//*****************The program.
//
deltat = t(2) - t(1);//on récupère la discrétisation du temps. Time discretisation deltat.

[k,l] = size(f);
n = d/deltat;
//cas particulier : le retard est un multiple de la discrétisation. Delay/deltat is an integer.  
if floor(n) == ceil(n) then
  tt = t;
  fd = [zeros(k,n),f];
  fd = fd(:,1:l);
else
//Cas général. Delay/deltat is not an integer : time step discretisation is not constant.
  i = floor(n);
  t1 = t(1:i+1);
  t2 = [d:deltat:t(l)];
  tt = [t1,t2];
  tt = tt(1:l);
  fd = [zeros(k,i+1),f];
  fd = fd(:,1:l);
end
endfunction


