// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
//This program gives partial fraction decomposition of a system or a rational.
// ====================================================================
function [elm, elm1] = frac_decomp(r)
//
//  ******************Tests des arguments d'entrée.Inputs tests.
//
if argn(2) > 1 then 
  error(msprintf(_("%s: Wrong number of input arguments %d: expected.\n"), "frac_decomp", 1))
end
if and(typeof(r) <> ["rational","zpk","state-space"]) | size(r,"*") > 1 then
  error(msprintf(_("%s: Wrong type for argument %d: Rational or SISO linear system expected.\n"), "frac_decomp", 1))
end
if typeof(r) == "state-space" then 
  r = clean(ss2tf(r));
end
if typeof(r) == "zpk" then 
  r = zpk2tf(r);
end
//  ******************Fin des tests des arguments d'entrée.
//
//  *****************Programme principal utilisant le programme mroots.sci.
//  *****************Main program mroots.sci is used.
num = r.num;den = r.den;
x = poly(0,varn(den));
[rest,elm1] = pdiv(r.num,r.den);
rm = mroots(den);
rac = clean(rm(:,1),1.e-14); mul = real(rm(:,2));//roots and multiplicities.
//Be care rm is complex but mul must be real !!
//  *****************Les racines réelles et multiplicités. Real roots and multiplicities.
k = find(imag(rac) == 0);
if k == [] then
  polr = []; DENr = []; Vr = [];
else
  racr = real(rac(k)); mulr = mul(k);
//  *****************Les polynômes élémentaires. 
//  *****************Polynomials with reals roots.
  polr = [];
  for j = 1:length(racr)
    racrj = racr(j); mulrj = mulr(j);
    polrj = (x - racrj)^(1:mulrj)';
    polr = [polr;polrj];
  end
  DENr = ones(polr)*den;
  Vr = ones(polr);
end
//  *****************Polynomials with complex roots.
k = find(imag(rac) > 0);
if k == [] then
  pol = polr; DEN = DENr; V = Vr;  
else
  raci = rac(k); muli = mul(k);
  poli = []; DENi = [];
  for j = 1:length(raci)
    racij = raci(j); mulij = muli(j);
    trino = real((x - racij)*(x - conj(racij)));
    polij = trino.^(1:mulij);
    polij = [polij;polij];
    polij = matrix(polij,2*mulij,1);
    poli = [poli;polij];
    DENij = [den;x*den];
    DENij = repmat(DENij,mulij,1);
    DENi = [DENi;DENij];
  end
  [lig,col] =size(DENi);
  l = [2*(1:lig/2)];
  Vi = ones(poli);
  Vi(l) = x;//For numerator of each element.
  pol = [polr;poli];
  DEN = [DENr;DENi];
  V = [Vr;Vi];
end
//  *****************Calcul de la matrice à transposer et inverser.
[R, Q] = pdiv(DEN,pol);//Q=numer(DEN./pol) 
Q = clean(Q);
A = coeff(Q,0:degree(den)-1)';//The matrix.
coerest = coeff(rest,0:degree(den)-1)';
Alpha = A\coerest;
//  *****************Présentation.
Alp = Alpha.*V;//The residues 
elm = Alp./pol;
l=find(degree(elm.num)==1);
if  l == [] then,
else
  elm(l-1,1)=elm(l-1,1)+elm(l,1);
  elm(l,1)=[];
end
endfunction














