// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
// We compute the gain margin of a SISO transfer function with delay.
// ====================================================================
function [gm, fgm, n] = g_marginrd(hd)
//
//*****************Tests des arguments d'entrée.Inputs arguments tests.
//
select typeof(hd)
  case "rational" then
    [gm, fgm] = g_margin(hd);
    return
  case "state-space" then
    h=clean(ss2tf(hd));
    [gm, fgm] = g_margin(h);
    return
  case "rd" then
    delay = hd.iodelay; h = hd.H;
  case "zpk"
    h=zpk2tf(hd);
    [gm, fgm] = g_margin(h);
    return    
  else
        error(97,1),
end;
if or(size(h)<>[1 1]) then
  error(msprintf(_("%s: Wrong size for input argument #%d: Single input, single output system expected.\n"),"g_marginrd",1))
end
//*****************Fin des tests des arguments d'entrée.End of tests.
//
//The problem is : find w=w(i) such as imag(num(jw)*den(-jw)*exp(j*d*w))=0.
//
w = poly(0,"w");//
//Now we normalise : if s--->d*s, w--->d*w and the time t--->t/d (Laplace theorem).
h = horner(h,s/delay);//with s1=s/delay, exp(-d*s)-->exp(-s) and hnew--->h(s/delay) ; now delay == 1.
h.dt = "c";
AA = horner(h.num,%i*w)*conj(horner(h.den,%i*w));//numerator of h.nuw(%i*w)/h.den(%i*w). 
A = real(AA);B = imag(AA);
dA = derivat(B) - A;dB = derivat(A) + B;
//Now we must find the roots of : B(w)*cos(w)-A(w)*sin(w),A and B are polynomials. 
//Initialisation of algorithm.
hd1 = iodelay(h,1)//With hnew.iodelay == 1.
hdpad = padedelay(hd1,4);//for initialisation we used Pade approximation of new iodelay system.
[gminit,fgminit] = g_margin(hdpad);//Gain margin of hdpad.
ws =2*%pi*fgminit//Frequency in rd/s.
//Now the loop with Newton_Raphson algorithm.
deff("[f] = ma_fonctf(x)", "f = horner(B,x).*cos(x) - horner(A,x).*sin(x)");
deff("[df] = ma_fonctdf(x)", "df = horner(dA,x).*cos(x) - horner(dB,x).*sin(x)");
function resultx = mynewt_raphs(x0)
  tol = 1.e-7;
  x1 = x0 - (ma_fonctf(x0)/ma_fonctdf(x0));
  while abs(x1 - x0) > tol
    x0 = x1;
    x1 = x0 - ma_fonctf(x0)/ma_fonctdf(x0);
  end
  resultx = x1;
endfunction
wl = mynewt_raphs(ws);
mingain = real(freqd(h.num,h.den,1,%i*wl));//Real amplitude of System at this frequency.
mingain = abs(mingain);
gm = -20*log(mingain)/log(10); //tranform into Db.
fgm = wl/(2*%pi*delay);//select frequency and take account delay.
endfunction
