// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
// We compute the modulus margin of a SISO transfer function.
// ====================================================================
function [mgp, fmgp] = m_margin(h)
//
//*****************Tests des arguments d'entrée.Inputs arguments tests.
//
select typeof(h)
 case "rational" then,
 case "state-space" then
   h = clean(ss2tf(h));
 case "zpk" then
   h = zpk2tf(h);
 else
   error(97,1);
end
if or(size(h) <> [1,1]) then
  error(msprintf(_("%s: Wrong size for input argument #%d: Single input, single output system expected.\n"),"m_margin",1));
end
//
//*****************Fin des tests des arguments d'entrée.End of tests.
//
invsi = 1 + h;//Inverse de la sensibilité.
si = 1/invsi;//La sensibilité.
fmgp = ffreson(si);//Maximum(s).//car bug freson, dans certains cas.
if fmgp == [] then
  error(msprintf(_("%s: No maximum for sensibility function of %dth input argument.\n"),"m_margin",1))
  return
end
fmgp = fmgp(find(fmgp <> 0));//il faut éliminer les fréquences nulles.
rep = repfreq(invsi,fmgp);
mgp = abs(rep);
[mgp,k] = min(mgp);//Marge de module.
fmgp = fmgp(k);//Fréquence correspondante.
endfunction


