// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/l icenses.en.html
// ====================================================================
// We compute the modulus margin of a SISO transfer function with delay.
// ====================================================================
function [mgp,fmgp] = m_marginrd(hd,f1,f2)
rhs = argn(2);
//*****************Tests des arguments d'entrée.Inputs arguments tests.
if and (rhs == [1,3]) then
  error(msprintf(_("%s: Wrong number for input arguments: %d or %d expected.\n"),"m_marginrd",1,3));
  return
end
select typeof(hd)
  case "rational" then
    [mgp, fmgp] = m_margin(hd);
    return
  case "zpk" then
    hd = zpk2tf(hd)
    [mgp, fmgp] = m_margin(hd);
    return
  case "state-space" then
    h = clean(ss2tf(hd));
    [mgp, fmgp] = m_margin(h);
    return
//*****************Systèmes continus avec retard. Continuous SISO systems with delay.
  case "rd" then 
  h = hd.H; delay = hd.iodelay;
  else
    error(97,1);
end
if or(size(h) <> [1,1]) then
  error(msprintf(_("%s: Wrong size for input argument #%d: Single input, single output system expected.\n"),"m_marginrd",1))
end
//*****************Fin des tests des arguments d'entrée.End of tests.
//
//*****************Programme principal. Main program.
//
//Initialisation of algorithm.
//Now we normalise : if s--->d*s, w--->d*w and the time t--->t/d (Laplace theorem).
h = horner(h,s/delay);//with s1=s/delay, exp(-d*s)-->exp(-s1) and hnew--->h(s/delay) ; newdelay == 1.
h.dt = "c";
hd=iodelay(h,1);
tol = 1.e-7;
hdpad = padedelay(hd,4);
//for initialisation we used Pade approximation of iodelay system.
[mpad,fmpad] = m_margin(hdpad);//Modulus margin of hdpad.
if rhs == 3 then 
  ff2 = f2/delay;ff1 = f1/delay;
else
  ff1 = 0.9*fmpad;ff2 = 1.1*fmpad;
end
repi = repfreqrd(hd,[ff1,ff2]);
repi =repi + ones(repi);
deltaL = abs(abs(repi(1)) - abs(repi($)));
if deltaL < tol then
  fmgp = fmpad/delay; mgp = mpad;
  return;
end
l = 25;
while deltaL >= tol
    fc = linspace(ff1,ff2,l);
    pas = fc(2) - fc(1);
    rep = repfreqrd(hd,fc);
    rep = rep + ones(rep);
    repc = abs(rep);
    [Lc,k] = min(repc);
    fmc = fc(k);
    if and([k > 1,k < l]) then
      ff1 = fc(k-1);
      ff2 = fc(k+1);
      Lc1 = repc(k-1); Lc2 = repc(k+1);    
      deltaL = abs(Lc1 - Lc2);
    else
      warning("It seems that upper frequency or lower frequency is not good.");
      return
    end
    fmgp = fmc;
end
if abs(repfreqrd(hd,fmgp)) >= 1 then
  error(msprintf(_("%s: No modulus margin : Instable system ?\n" ),"m_marginrd"))
  return  
end
fmgp = fmgp/delay;
mgp = Lc;
endfunction   
