// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
//Cette fonction renvoie, pour un vecteur colonne de systèmes linéaires continus, 
//une matrice "va", de dimension (n,k) : k représentant les k premiers 
//moments pondérés,par "a",des réponses impulsionnelles H(t) soit :
//intégrale de 0 à infinie de exp(-at)*t^k*dt, du système SIMO,"sys".
//C'est le développement de Taylor de "sys" autour de "s = -a".
//"n" est la dimension de "sys".
//This function returns, for continuous systems, a matrix "va",
//the size of which is (n,k) : k represents the first generalised moments
//of impulse response H(t).
//It is Taylor's developpment of "sys" near "s = -a".
//"n" is the dimension of "sys".
// ====================================================================
function va = mompon(sys, a, k)
rhs = argn(2);
if type(sys) <> 16 | sys.dt <> "c" then
   error(msprintf(_("%s: Wrong type for input argument #%d: Continuous time linear system expected.\n"),"mompon",1));
   return
end
if sys(1)(1) == "lss" then 
   sys = clean(ss2tf(sys));
end
simp_mode(%t);//Au cas où sys = syslin("c",num,den) avec un terme commun entre "num" et den".
sys = simp(sys);//On force la simplification.
select rhs
 case 1 then
     error(msprintf(_("%s: Wrong number of inputs arguments %d: 2 or 3 expected.\n"),"mompon"));
 case 2 then
     k = int(a);
     a = 0;
 case 3 then
     k = int(k);
     if type(a) <> 1 then
        error(msprintf(_("%s: The argument k must be a integer, and a must be a real number.\n"),"mompon"));
     end
end
if k <= 0 then
   error(msprintf(_("%s: The last input argument k must be a positive integer.\n"),"mompon"));
end
var = varn(sys.den);
var = poly(0,"var");
num = sys.num;
den = sys.den;
//test sur les degrés de num et den, (degree(num)-degree(den)+1) <= 0.
k1 = length(num);
 for i = 1:k1
     numi = num(i)
     deni = den(i)
     if degree(numi) >= degree(deni) then
        error(msprintf(_("%s: One system is not predictable.\n"),"mompon"));
     end
 end
num1 = horner(num,(var+a));
den1 = horner(den,(var+a));
//test sur existence ou pas de pôle en "-a" pour un des systèmes.
 for i = 1:k1
   den1i = den1(i)
     if abs(freq(den1i,1,-a)) <= 1.e-10 then
        error(msprintf(_("%s: a is a pole for one system.\n"),"mompon"))
        return
     end
 end
num11 = horner(num1,1/var);
den11 = horner(den1,1/var);
solu = ldiv(num11.num,den11.num,k)';
//présentation de la solution.
va = [];
 for j = 1:k1
     va = clean([va;solu((j-1)*k+1:j*k)]);
 end
endfunction








