// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2017.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
// Lieux (et pseudo-lieux) de Nyquist de systèmes classiques, avec retard pur, ou lieux définis par trois vecteurs (matrices) constants.
// On utilise la fonction [fr,db,phi]=dbphifr( ), en particulier pour tracer les pseudo-lieux, ou dpf_cd.sci.
// DEPENDENCIES : only dbphifr.sci or dpf_cd.sci (Autoelem Toolbox) and iodelay toolbox for input/output delays systems.
// ====================================================================
function nnyquist(varargin)
rhs = size(varargin);
sys = varargin(1);
//cacsdlib.nyquist
if and(typeof(sys) <> ["state-space","rational","zpk","rd","constant"]) then
  msg = _("Wrong type for argument #%d: rational, state-space, iodelay system or 3 matrices of constants expected.\n");
  error(msprintf(msg, 1))
  return;
end
if type(varargin(rhs)) == 4 then
  symetry = varargin(rhs);
  rhs = rhs-1;
else
  symetry = %f; 
end

if rhs==1 then //Un système seul est envisagé.
   if type(sys) <> 1 then
     [frq,db,phi] = dbphifr(sys,1d-3,1d3);
     nyquist(frq,db,phi,symetry);
   end
else
  if typeof(sys) == "constant" then
    if rhs < 3 then
      error(msprintf(_("%s: Wrong size for inputs arguments #%d: 3 matrices of constants expected.\n"),"nyquist",1));
    end
    nyquist(varargin(:),symetry);
  else
    der = varargin(rhs);
    avder = varargin(rhs-1);
    if type(der) == 10 & type(avder) == 10 then //Le dernier argument et l'avant dernier sont des chaînes de caractères.
      comments = der;//Le dernier argument est pris pour une légende.
      [frq,db,phi] = dbphifr(varargin(1:rhs-1));
      nyquist(frq,db,phi,comments,symetry);
    elseif type(der) == 10 & type(avder) <> 10 then //Ici le dernier argument est soit une légende soit un vecteur d'angles.
      [teta,ieer] = evstr(der); //Test pour savoir si c'est une légende ou un vecteur de réels.
      if ieer == 0 & type(teta) == 1 then //L'évaluation est correcte, et c'est un vecteur de réels.
        [frq,db,phi] = dbphifr(varargin(1:rhs));
        nyquist(frq,db,phi,symetry);
      else //Ce n'est pas un vecteur de réels.
        comments = der; //On le prend pour une légende.
        [frq,db,phi] = dbphifr(varargin(1:rhs-1));
        nyquist(frq,db,phi,comments,symetry);
      end
    else //Pas de légende, pas de vecteur de scalaires. 
      [frq,db,phi] = dbphifr(varargin(1:rhs));
      nyquist(frq,db,phi,symetry);
    end
  end
end
//rajouter des éléments graphiques
fig = gcf();
fig.background = 8;
fig.children.grid = [13,13];
fig.children.y_label.text = "Partie imaginaire";
fig.children.x_label.text = "Partie réelle";
xpoly([-1.2;-0.8],[0;0]);//pour repérer le point -1. 
xpoly([-1;-1],[-0.2;0.2]);//pour repérer le point -1.
a = gca();
a.title.text = "Courbe de Nyquist";a.title.font_style = 4;a.title.font_size = 2;
CH = a.children;
if CH(3).type == "Legend" then
  CH(3).legend_location = "in_upper_left";
end
CH(1:2).foreground=24;

endfunction


