// Copyright (C) 2010-2011 - INRIA - Serge Steer.
// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
//compute the phase margin of a SISO transfer function with i/o delay.
// ====================================================================
function [phm, fr] = p_marginrd(hd)
select typeof(hd)
  case "rational" then
    [phm, fr] = p_margin(hd);
    return
  case "zpk" then
    hd = zpk2tf(hd);
    [phm, fr] = p_margin(hd);
    return
  case "state-space" then
    h = clean(ss2tf(hd));
    [phm, fr] = p_margin(h);
    return
  case "rd" then
    delay = hd.iodelay; h=hd.H;
else
      error(97,1)
end
//*****************
if or(size(h) <> [1 1]) then
  error(msprintf(_("%s: Wrong size for input argument #%d: Single input, single output system expected.\n"),"p_marginrrd",1))
end
eps = 1.e-7;// threshold used for testing if complex numbers are real or pure imaginary
w = poly(0,"w");
niw = horner(h.num,%i*w);
diw = horner(h.den,%i*w);
// |n(iw)/d(iw)|=1 --> (n(iw)*n(-iw))/(d(iw)*d(-iw))=1 -->  (n(iw)*n(-iw)) - (d(iw)*d(-iw))=0
// For iodelay system  |n(iw)/d(iw)|=1 is the same as system without delay.
w = roots(real(niw*conj(niw) - diw*conj(diw)),"e");
//select positive real roots.
ws = real(w(find((abs(imag(w)) < eps) & (real(w) >0)))); //frequency points with unitary modulus.
if ws == [] then
  phm = [];
  fr = [];
  return
end
f = horner(h,%i*ws);
phi = atand(imag(f),real(f)) - (180/%pi)*delay*ws;// phase of the frequency response (in [-180,180]).
//avoid near 0 negative phases that will give phm=180 instead of -180.
phi(phi>-1e-12&phi<0) = 0;
//compute the margins.
phm = pmodulo(phi,360) - 180;
//select the min value together with associated frequency in Hz
[w,k] = min(abs(phm));
phm = phm(k)
fr = ws(k)/(2*%pi);
endfunction
