// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
//On commence par calculer un approximant du retard pur exp(-s). Puis exp(-d*s) (d matrice des retards) avec horner.
//Ensuite on combine la matrice des systèmes sans retard avec la matrice des approximants de Padé.
//Si le troisième argument d'entrée est un caractère alors on calcule respectivement les approximants (de degré np ou 2*np) de Laguerre (degré np), 
//de Kautz (degré 2*np) ou de Pade du deuxième ordre (degré 2*np).
// ====================================================================
//The first time we compute one pade (np,mp) approximation of exp(-s). After we compute exp(-d*s) (d matrix of delays) by horner function.
//If the type of the third input argument is a string ,we compute Laguerre, Kautz, or Padé second order approximation if flag="L" or "K" or "P".
//After that we give total approximation ; system and approximation of delay in serie.
// ====================================================================
function hpade = padedelay(hd, np, mp) //or padedelay(hd, np) or padedelay(hd, np, flag).
//
//*****************Tests des arguments d'entrée nombre et type. Input arguments tests.
//
rhs = argn (2);
if rhs < 2 | rhs > 3  then
  error(msprintf(_("%s: Wrong number for input arguments: %d to %d expected.\n"),"padedelay",2,3));
end
if typeof(hd) <> "rd" then//Test pour savoir si hd est un système à retard.
  error(msprintf(_("%s: Wrong type for input argument #%d: delayed transfer function expected.\n"),"padedelay",1));
end
if type(np) <> 1 then
  error(msprintf(_("%s: Wrong type for input argument #%d: integer expected.\n"),"padedelay",2));
end  
d = hd.iodelay;//Le retard pur.Delay
h = hd.H;//Le système.The system without delay.
va = varn(h.den);//La variable du système (chaîne de caractères).
var=poly(0,va);
if rhs == 2 then
   mp = np; 
end
if size(np) <> size(d) then
  error(msprintf(_("%s: Wrong size for input arguments #%d expected.\n"),"padedelay",2));
end
//
//*****************Fin des tests des arguments d'entrée.End of input arguments tests.
//
//*****************Calcul de pade(exp(-x),np,mp).Calcul de la matrice. Value of pade(exp(-x),np,mp).
//
//on prendra np et mp inférieurs à 10, car pour np et mp = 12 par exemple, le dernier coefficient (degré 12) vaut : 7.720D-16
//proche de %eps qui vaut 2.220D-16.(Voir aussi les publications sur ce sujet).
  [lignd,colond] = size(d);
  sl = [];
  for i=1:lignd
    sli = [];
    for j=1:colond
      dij = d(i,j);
      npij = np(i,j);     
      if type(mp) == 1 then
          if size(mp) <> size(d) then
            error(msprintf(_("%s: Wrong size for input arguments #%d expected.\n"),"padedelay",3));
          end
        mpij = mp(i,j);
        slij = padexp(npij,mpij);
      else
//
//*****************Les approximants de Laguerre ou de Kautz ou de Pade du deuxième ordre. Laguerre, Kautz, second order Pade. 
//
        flag = part(mp,1);
          select flag
            case "L" then //Laguerre approximation.
              slij = syslin('c',(1-var/(2*npij))**npij,(1+var/(2*npij))**npij);
            case "K" then //Kautz approximation.
              slij = syslin('c',(1-var/(2*npij)+(1/2)*(var/(2*npij))**2)**npij,..
              (1+var/(2*npij)+(1/2)*(var/(2*npij))**2)**npij);
            case "P" then //Pade (order two) approximation.
              slij = syslin('c',(1-var/(2*npij)+(1/3)*(var/(2*npij))**2)**npij,..
              (1+var/(2*npij)+(1/3)*(var/(2*npij))**2)**npij);
            else
              error(46);
              return
          end
      end
//
//*****************Matrice des approximants en tenant compte de "d".
//
      slij = horner(slij,dij*var); 
      sli = [sli,slij];
    end
    sl = [sl;sli];
  end
//
//*****************Fin des approximents de Padé (np,mp) classiques  de Laguerre, de Kautz, de Pade du second ordre de exp(-ds).
//
//*****************La matrice complète.
hpade = h.*sl;
hpade = clean(hpade);
//On risque d'avoir un hpade d'ordre inférieur à la somme : n + (ordre de h1) :
//on ne conserve que les coefficients qui sont supérieurs à 1.d-10. 
endfunction
//
//*****************Fonction intermédiaire : calcul de l'approximant de Padé de exp(-x). Pade of exp(-x).
//
function sl = padexp(np,mp)
  b(1) = 1;
//dénominateur.
  coeden = [b(1)];
  for j=1:np
    b(j+1) = ((np-j+1)/(np+mp-j+1))*(b(j)/j);
    coeden = [coeden,b(j+1)];
  end
  den = poly(coeden,va,"c");
//numérateur
  if mp == np then
    num = horner(den,-var);
  else
    coenum = [b(1)];
    for j=1:mp
      b(j+1) = -((mp-j+1)/(np+mp-j+1))*(b(j)/j);
      coenum = [coenum,b(j+1)];
    end
    num = poly(coenum,va,"c");
  end  
   sl = syslin("c",num/den);
endfunction

