// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
//Ce programme est une liste d'appel et une présentation des
//paramètres de Markov sous forme de lignes : (1 ligne, un système).
//C'est la même présentation que celle vue pour moments pondérés.
//C'est le développement de Taylor de "sys" pour "s = %inf".
//
//This program presents Markov's parameters in rows : (one row, one system).
//It is the same presentation as moments (function "mompon").
//It is Taylor's development of "sys" at "s = %inf".
function ma = pmark(sys, k)
if type(sys) <> 16 | sys("dt") <> "c" then
   error(msprintf(_("%s: Wrong type for input argument #%d: Continuous time linear system expected.\n"),"pmark",1));
   return
end
if sys(1)(1) == "lss" then
 sys = clean(ss2tf(sys));
end
simp_mode(%t); //Au cas où sys = syslin("c",num,den) avec un terme commun entre "num" et den".
sys = simp(sys); //On force la simplification.
num = sys.num;
den = sys.den;
k1 = length(num);
eps = 1.e-14;
solu=[];
for i = 1:k1
 numi = num(i);
 deni = den(i);
 if degree(numi)>=degree(deni) then
    error(msprintf(_("%s: One system is not predictable.\n"),"pmark"));
 end
 if horner(numi,0) == 0 then
    error(msprintf(_("%s: No Markov parameters.\n"),"pmark"));
 end
 coedi = coeff(deni); //voyons s'il y a des intégrateurs si oui, les premiers paramètres
 ndi = min(find(coedi <> 0))-1; //sont nuls : autant de paramètres nuls que d'intégrations dans "sys".
 coei = coedi(ndi+1:$);
 var = varn(deni); //on construit un dénominateur qui n'a pas de racines nulles.
 denci = poly(coei,var,"c");
 if k < ndi then
    error(msprintf(_("%s: Increase k value.\n"),"pmark"));
 end
 solui = ldiv(numi,denci,k-ndi)';
 if ndi <> 0 then  //on rajoute les premiers paramètres de Markov qui sont nuls.
    solui = [zeros(1,ndi),solui];
 end
 solu = [solu;solui];
end
ma = solu; //présentation de la solution
endfunction




