// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
// Thiran filter for delay.
// ====================================================================
function hz = thiran(d, dom, flag)
rhs = argn(2);
//*****************Tests des arguments d'entrée.Tests for input arguments.
if rhs < 2 | rhs >3  then
    error(msprintf(_("%s: Wrong number of input arguments: %d to %d expected.\n"),"thiran",2,3))
end    
if type(dom) <> 1 | dom <= 0 then
    error(msprintf(gettext("%s: Wrong type for input argument #%d: A real positive number expected.\n"),"thiran",2));
end
if size(dom,'*') <> 1 then
    error(msprintf(gettext("%s: Wrong size for input argument #%d: A scalar expected.\n"),"thiran",2));
end
if type(d) <> 1 | d < 0 then
    error(msprintf(gettext("%s: Wrong type for input argument #%d: A real positive number expected.\n"),"thiran",1));
end
if size(d,'*') <> 1 then
    error(msprintf(gettext("%s: Wrong size for input argument #%d: A scalar expected.\n"),"thiran",1));
end
//*****************Fin des tests des arguments d'entrée.End of tests.
//
//****************Cas particulier : retard pur considéré comme nul. Particular case : Delay == 0.
z = %z;tol = 1.e-10;
dsurdom = tol*round((d/dom)/tol);
if d == 0 then
   unz = poly(1,"z","c");
   hz = syslin(dom, unz, unz);
   return
end
//*****************Fin cas particulier. End of particular case.
//
//*****************Cas général n°1 (avec "flag == "y"). Case n° 1, "flag == "y...".
if rhs == 3 then 
//Si flag="yes" : "de" est la partie entière inférieure de "d/dom" et donnera dans "hz" un terme en
// "z**(-de)" le reste du retard étant intégré dans "hz" sous forme d'un terme du premier ordre.
   if part(flag,1) == "y" then
     de = floor(dsurdom); //Partie entière inférieure du rapport (retard pur)/(période d'échant).
     dr = dsurdom-de; //Le reliquat.
     coen = [1,(1-dr)/(1+dr)]; //Le filtre de Thiran du premier ordre correspondant au reliquat.
     num = poly(coen,"z","c");
     coed = [coen($),1];
     den = z**de*poly(coed,"z","c");
     hz = syslin(dom,num/den); //Le fitre total.
   else
     error(msprintf(gettext("%s: Last input argument #%d must be the string yes or ye or y.\n"),"thiran",3));
   end
//*****************Fin cas général n°1.  
else
//*****************Cas général n°2.(avec "flag <> "y").  Case n° 2, "flag <> "y...".
   m = ceil(dsurdom);
   d=dsurdom;
   n = 0:m-1;
   coen = cumprod([1,(m-n).*(m-n-d)./((n+1).*(n+1+d))]);
   num = poly(coen,"z","c");
   coed = coen($:-1:1);
   den = poly(coed,"z","c");
   hz = syslin(dom,num/den); //Filtre de Thiran normal.
//*****************Fin cas général n°2.
end
endfunction
